% slui --- Simulink user interface.
%
%      slui
%
% This is an example for the interaction between a Simulink model and
% a Matlab graphical user interface.

%% slui.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of APSE, the "Advanced Programming in the Simulink
% Environment" lecture notes.

% APSE is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free
% Software Foundation, either version 3 of the license, or (at your
% option) any later version.

% APSE is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
% for more details.

% You should have received a copy of the GNU General Public License
% along with APSE.  If not, see <http://www.gnu.org/licenses/>.

% Additional permission under GNU GPL version 3 section 7

% If you modify APSE, or any covered work, by linking or combining it
% with MATLAB from The Mathworks, Inc. (or a modified version of MATLAB),
% containing parts covered by the same license as MATLAB, the licensors
% of APSE grant you additional permission to convey the resulting work.

%% Code:

% Program entry point.
function [sys, X0, str, t] = slui(tau, X, U, flag)

  switch nargin
   case 4
    % S-function call-back.
    switch flag
     case 0
      % Initialization.
      [sys, X0, str, t] = slui_init(gcbh);
     case 1
      % Calculate derivatives.
      sys = [];
     case 2
      % Update discrete states.
      sys = slui_update(gcbh, U);
     case 3
      % Calculate outputs.
      sys = [];
     case 4
      % Get time of next sample hit.
      sys = [];
     case 9
      % Termination.
      sys = [];
     case 'start'
      slui_start(gcbh);
     case 'stop'
      slui_stop(gcbh);
     case 'clear'
      slui_clear(gcbh);
     case 'install'
      slui_install(gcbh);
     otherwise
      error('Invalid argument');
    end
   case 0
    % Command line interface.
    slui_open;
   otherwise
    error('Wrong number of arguments');
  end

% S-function initialization.
%
% Argument BL is the Simulink block handle.
function [sys, X0, str, t] = slui_init(bl)

  % Clear user data structure.
  slui_clear(bl);

  % Define S-function attributes.
  S = simsizes;

  S.NumInputs = 3;
  S.NumOutputs = 0;
  S.NumContStates = 0;
  S.DirFeedthrough = 0;
  S.NumSampleTimes = 1;

  sys = simsizes(S);

  % Initial values.
  X0 = [];

  % Always empty (reserved for future use).
  str = [];

  % Sample times (0 means continuous sample
  % time, -1 means inherited sample time, and
  % -2 means variable sample time).
  t = [-1, 0];

% Start simulation.
%
% Argument BL is the Simulink block handle.
function slui_start(bl)

  % Show user interface.
  fig = slui_open;
  slui_lock(fig);

  % Save widget handles.
  h = get(fig, 'UserData');
  set_param(bl, 'UserData', h);

  % Reset trajectory objects.
  set(h.fig, 'DoubleBuffer', 'off');

  set(h.tail, 'EraseMode', 'none', ...
              'XData', [], ...
              'YData', [], ...
              'ZData', []);

  set(h.head, 'EraseMode', 'xor', ...
              'XData', [], ...
              'YData', [], ...
              'ZData', []);

  % Initialize variable data.
  %
  % Please note that a pair of get/set calls for a Handle Graphics
  % object (or getappdata/setappdata) is usually faster than a pair
  % of get_param/set_param calls for a Simulink block.
  size = get(h.size, 'UserData');
  iter = get(h.iter, 'UserData');

  set(h.ax, 'UserData', struct('elem', 0, ...
                               'size', size, ...
                               'buf', zeros(size, 3), ...
			       'count', 0, ...
			       'iter', iter));

  % Update the screen.
  drawnow;

% Update graphics.
%
% First argument BL is the Simulink block handle.
% Second argument U is the input signal vector.
function sys = slui_update(bl, U)

  % Return value.
  sys = [];

  % Get widget handles.
  h = get_param(bl, 'UserData');

  % Get variable data.
  var = get(h.ax, 'UserData');

  % Grow data buffer.
  if var.elem < var.size
    var.elem = var.elem + 1;
  end

  % Tail indices.
  ind = 2:var.elem;

  % Shift tail coordinates.
  if ~ isempty(ind)
    var.buf(ind, :) = var.buf(ind - 1, :);
  end

  % Add head coordinates.
  var.buf(1, :) = U;

  % Update trajectory objects.
  if var.elem > 2
    set(h.tail, 'XData', var.buf(2:3, 1), ...
                'YData', var.buf(2:3, 2), ...
                'ZData', var.buf(2:3, 3));
  end

  set(h.head, 'XData', var.buf(1, 1), ...
              'YData', var.buf(1, 2), ...
              'ZData', var.buf(1, 3));

  % Update coordinates after every VAR.ITER loops.
  if rem(var.count, var.iter) == 0
    set(h.x, 'String', sprintf('%.5G', var.buf(1, 1)));
    set(h.y, 'String', sprintf('%.5G', var.buf(1, 2)));
    set(h.z, 'String', sprintf('%.5G', var.buf(1, 3)));

    var.count = 1;
  else
    var.count = var.count + 1;
  end

  % Save variable data.
  set(h.ax, 'UserData', var);

  % Update the screen.
  drawnow;

% Stop simulation.
%
% Argument BL is the Simulink block handle.
function slui_stop(bl)

  % Get widget handles.
  h = get_param(bl, 'UserData');

  % Get variable data.
  var = get(h.ax, 'UserData');

  % Tail indices.
  ind = 1:var.elem;

  % Draw trajectory objects exactly.
  set(h.fig, 'DoubleBuffer', 'on');

  set(h.tail, 'EraseMode', 'normal', ...
              'XData', var.buf(ind, 1), ...
              'YData', var.buf(ind, 2), ...
              'ZData', var.buf(ind, 3));

  set(h.head, 'EraseMode', 'normal', ...
              'XData', var.buf(1, 1), ...
              'YData', var.buf(1, 2), ...
              'ZData', var.buf(1, 3));

  % Final coordinates.
  set(h.x, 'String', sprintf('%.5G', var.buf(1, 1)));
  set(h.y, 'String', sprintf('%.5G', var.buf(1, 2)));
  set(h.z, 'String', sprintf('%.5G', var.buf(1, 3)));

  % Clear variable data.
  set(h.ax, 'UserData', []);
  slui_unlock(h.fig);

  % Update the screen.
  drawnow;

% Clear user data.
%
% Argument BL is the Simulink block handle.
function slui_clear(bl)

  data = get_param(bl, 'UserData');
  if ~ isempty(data)
    set_param(bl, 'UserData', []);
  end

% Install call-back functions.
%
% Argument BL is the Simulink block handle.
function slui_install(bl)

  % Call-back template.
  callback = sprintf('%s([], [], [], ''%%s'');', mfilename);

  set_param(bl, 'LoadFcn', sprintf(callback, 'clear'), ...
                'CopyFcn', sprintf(callback, 'clear'), ...
                'StartFcn', sprintf(callback, 'start'), ...
                'StopFcn', sprintf(callback, 'stop'));

  set_param(get_param(bl, 'Parent'), 'OpenFcn', 'slui');

% Open user interface.
%
% Only allow one instance of the user interface to exist.
function fig = slui_open

  tem = get(0, 'ShowHiddenHandles');
  set(0, 'ShowHiddenHandles', 'on');
  fig = findobj(0, 'Type', 'figure', ...
                   'Tag', '-*- slui -*-');
  set(0, 'ShowHiddenHandles', tem);

  if numel(fig) > 1
    delete(fig);
    fig = [];
  end
  if isempty(fig)
    fig = slui_create;
  else
    set(fig, 'Visible', 'on');
    figure(fig);
  end

% Create user interface.
function fig = slui_create

  % Load block diagram.
  [dia, bl] = slui_system;

  % Create graphical user interface.
  fig = mtk('figure', 0, '0x0', ...
            'Tag', '-*- slui -*-', ...
            'Name', slui_title, ...
            'CloseRequestFcn', {@slui_handler, 'close'}, ...
            'Resize', 'off', ...
            'Visible', 'off');

  mtk('render', fig, ...
      eval(mtk('read', fullfile(slui_prefix, 'slui.mtk'))));

  % Get widget handles.
  h = guihandles(fig);
  h.fig = fig;

  % Create trajectory objects.
  h.tail = line('Parent', h.ax, ...
                'Color', 'blue',  ...
                'LineStyle', '-', ...
                'LineWidth', 1, ...
                'XData', [], ...
                'YData', [], ...
                'ZData', []);

  h.head = line('Parent', h.ax, ...
                'Color', 'red',  ...
                'Marker', 'o', ...
                'MarkerEdgeColor', 'black', ...
                'MarkerFaceColor', 'red', ...
                'MarkerSize', 6, ...
                'XData', [], ...
                'YData', [], ...
                'ZData', []);

  % Fine tune axes.
  set(fig, 'CurrentAxes', h.ax);

  set(get(h.ax, 'XLabel'), 'String', 'x');
  set(get(h.ax, 'YLabel'), 'String', 'y');
  set(get(h.ax, 'ZLabel'), 'String', 'z');

  set(h.ax, 'XLim', [-25, 25], ...
            'YLim', [-25, 25], ...
            'ZLim', [0, 50]);

  grid(h.ax, 'on');
  view(h.ax, 3);

  axis(h.ax, 'vis3d');
  rotate3d(fig, 'on');

  % Save object handles.
  set(fig, 'UserData', h);

  % Merge Simulink parameters.
  set(h.size, 'String', get_param(bl, 'buffer_size'));
  slui_handler(h.size, [], 'size', 0);

  set(h.iter, 'String', get_param(bl, 'decimation'));
  slui_handler(h.iter, [], 'iter', 0);

% Lock the figure window.
function slui_lock(fig)

  % Disable destruction of figure window.
  val = getappdata(fig, 'slui_lock');
  if isempty(val)
    % Save figure poperties.
    setappdata(fig, 'slui_lock', get(fig));
    % Protect figure window.
    set(fig, 'CloseRequestFcn', []);
  end

  % Disable user interaction of widgets.
  h = get(fig, 'UserData');

  set(h.size, 'Enable', 'off');
  set(h.iter, 'Enable', 'off');

% Unlock the figure window.
function slui_unlock(fig)

  % Enable destruction of figure window.
  val = getappdata(fig, 'slui_lock');
  if ~ isempty(val)
    % Clear lock data.
    rmappdata(fig, 'slui_lock');
    % Restore figure poperties.
    set(fig, 'CloseRequestFcn', val.CloseRequestFcn);
  end

  % Enable user interaction of widgets.
  h = get(fig, 'UserData');

  set(h.size, 'Enable', 'on');
  set(h.iter, 'Enable', 'on');

% Load block diagram.
function [dia, bl] = slui_system

  dia = 'slui_model';
  load_system(dia);

  % Sanity check.
  bl = find_system(get_param(dia, 'Handle'), 'BlockType', 'S-Function', 'FunctionName', mfilename);
  if numel(bl) ~= 1
    error(sprintf('The Simulink model ''%s'' associated with Slui seems to be corrupted', dia));
  end

% Return installation directory.
function dir = slui_prefix

  persistent d;
  if isempty(d)
    d = fileparts(which(mfilename));
  end

  dir = d;

% Return corporate window title.
function str = slui_title(tag, fig)

  str = 'Slui';

  if nargin > 0 & numel(tag) > 0
    str = [str, ' - ', tag];
  end

  if nargin > 1
    set(fig, 'Name', str);
  end

% Process a call-back command.
function slui_handler(obj, ev, command, varargin)

  % Load block diagram.
  [dia, bl] = slui_system;

  % Call-back figure handle.  The 'gcbf' function does not work
  % in conjunction with explicit invocations of this function as
  % performed in 'slui_create' above.
  fig = get(obj, 'Parent');
  if fig == 0
    fig = obj;
  end

  % Widget handles.
  h = get(fig, 'UserData');

  try
    switch command
     case 'start'
      switch get_param(dia, 'SimulationStatus')
       case 'stopped'
        set_param(dia, 'SimulationCommand', 'start');
       case 'paused'
        set_param(dia, 'SimulationCommand', 'continue');
      end

     case 'pause'
      switch get_param(dia, 'SimulationStatus')
       case 'running'
        set_param(dia, 'SimulationCommand', 'pause');
       case 'paused'
        set_param(dia, 'SimulationCommand', 'continue');
      end

     case 'stop'
      switch get_param(dia, 'SimulationStatus')
       case 'running'
        set_param(dia, 'SimulationCommand', 'stop');
       case 'paused'
        set_param(dia, 'SimulationCommand', 'stop');
      end

     case 'size'
      size = slui_eval(get(h.size, 'String'), 'trajectory length');
      if isempty(size) | ~ slui_scalar_p(size) | rem(size, 1) ~= 0 | size <= 0
        set(h.size, 'String', mat2str(get(h.size, 'UserData')));
        error('Trajectory length has to be a positive integral number');
      end
      set(h.size, 'UserData', size);
      if isempty(varargin) & ishandle(bl)
        set_param(bl, 'buffer_size', mat2str(size));
      end

     case 'iter'
      iter = slui_eval(get(h.iter, 'String'), 'decimation');
      if isempty(iter) | ~ slui_scalar_p(iter) | rem(iter, 1) ~= 0 | iter <= 0
        set(h.iter, 'String', mat2str(get(h.iter, 'UserData')));
        error('Decimation has to be a positive integral number');
      end
      set(h.iter, 'UserData', iter);
      if isempty(varargin) & ishandle(bl)
        set_param(bl, 'decimation', mat2str(iter));
      end

     case 'close'
      try
        if strcmp(get_param(dia, 'Dirty'), 'on')
          if mtk('question', slui_title('Question'), ...
                 'Save Slui model before closing?')
            save_system(sys);
          end
        end
        close_system(sys, 0);
      end
      delete(fig);

     case 'help'
      mtk('message', slui_title('Help'), ...
	  sprintf(['This example runs a Simulink model in the\n', ...
		   'background and displays the simulation\n', ...
		   'results in the graphical user interface.\n', ...
		   '\n', ...
		   'Author: Ralph Schleicher <rs@ralph-schleicher.de>\n', ...
		   '\n', ...
		   'This program is part of the "Advanced\n', ...
		   'Programming in the Simulink Environment"\n', ...
		   'lecture notes.']));
    end
  catch
    mtk('error');
  end

% Remove leading and trailing whitespace characters.
function str = slui_nonblank(str)

  ind = find(~ isspace(str));
  if isempty(ind)
    str(1:end) = [];
  else
    str = str(min(ind):max(ind));
  end

% Evaluate a string in the Matlab workspace.
function val = slui_eval(str, tag)

  str = slui_nonblank(str);

  if isempty(str)
    val = [];
  else
    try
      val = evalin('base', str);
    catch
      if nargin < 2
        tag = 'Matlab expression';
      end
      error(sprintf('Invalid %s.\n\n%s', tag, lasterr));
    end
  end

% Return non-zero if A is a scalar.
function p = slui_scalar_p(a)

  if numel(a) == 1 & isnumeric(a) & imag(a) == 0
    p = 1;
  else
    p = 0;
  end

%% slui.m ends here
